from ZPublisher.HTTPResponse import HTTPResponse,accumulate_header
import sys
import re
class ModPythonHTTPResponse(HTTPResponse):
    """ A ZPublisher HTTPResponse object that uses mod_python request object
        instead of stdin.
    """
    def __init__(self,mod_python_request,body='',status=200,headers=None,stderr=sys.stderr,):
        '''\
        Creates a new response. In effect, the constructor calls
        "self.setBody(body); self.setStatus(status); for name in
        headers.keys(): self.setHeader(name, headers[name])"
        '''
        if headers is None:
            headers = {}
        self.headers = headers
        self.multi_headers = []

        if status == 200:
            self.status = 200
            self.errmsg = 'OK'
            headers['status'] = "200 OK"
        else:
            self.setStatus(status)
        self.base = ''
        if body:
            self.setBody(body)
        self.cookies = {}
        self.mod_python_request = mod_python_request
        self.stderr = stderr
        #Compatibility
        self.stdout=mod_python_request

    def retry(self):
        """Return a response object to be used in a retry attempt
        """

        # This implementation is a bit lame, because it assumes that
        # only stdout stderr were passed to the constructor. OTOH, I
        # think that that's all that is ever passed.

        return self.__class__(self.mod_python_request,stderr=self.stderr)

    def setHeader(self, name, value, literal=0):
        '''\
        Sets an HTTP return header "name" with value "value", clearing
        the previous value set for the header, if one exists. If the
        literal flag is true, the case of the header name is preserved,
        otherwise word-capitalization will be performed on the header
        name on output.'''
        name = str(name)
        value = str(value)
        key = name.lower()
        if accumulate_header(key):                          
            self.multi_headers.append((name,value))
            return
        name = literal and name or key
        self.headers[name] = value

    def addHeader(self, name, value):
        '''\
        Set a new HTTP return header with the given value, while retaining
        any previously set headers with the same name.'''
        name = str(name)
        value = str(value)
        self.multi_headers.append((name,value))

    def fixHeader(self,key,val):
        if key.lower() == key:
            # only change non-literal header names
            key = "%s%s" % (key[:1].upper(), key[1:])
            start = 0
            l = key.find('-',start)
            while l >= start:
                key = "%s-%s%s" % (key[:l],key[l+1:l+2].upper(),key[l+2:])
                start = l + 1
                l = key.find('-', start)
        return (key,val)
        
    def __str__(self,
                html_search=re.compile('<html>',re.I).search,
                ):
        if self._wrote:
            return ''       # Streaming output was used.

        headers = self.headers
        body = self.body

        if not headers.has_key('content-length') and \
                not headers.has_key('transfer-encoding'):
            self.setHeader('content-length',len(body))

        headersl = []
        append = headersl.append

        # status header must come first.
        append("Status: %s" % headers.get('status', '200 OK'))
        append("X-Powered-By: Zope (www.zope.org), Python (www.python.org)")
        if headers.has_key('status'):
            del headers['status']
        for key, val in headers.items():
            key, val = self.fixHeader(key, val)
            append("%s: %s" % (key, val))
        if self.cookies:
            headersl = headersl+self._cookie_list()
        for key, val in self.multi_headers:
            key, val = self.fixHeader(key, val)
            append("%s: %s" % (key, val))        
        append(body)
        return '\n'.join(headersl)
    
    def outputBody(self):
        """Output the response body"""
        if self._wrote:
            return ''       # Streaming output was used.

        headers = self.headers
        body = self.body

        if not headers.has_key('content-length') and \
                not headers.has_key('transfer-encoding'):
            self.setHeader('content-length',len(body))

        headersl = []
        append = headersl.append

        # status header must come first.
        self.mod_python_request.headers_out["Status"]=headers.get('status', '200 OK')
        status = int(headers.get('status', '200 OK').split()[0])
        self.mod_python_request.status = status
        self.mod_python_request.headers_out["X-Powered-By"]="Zope (www.zope.org), Python (www.python.org)"
        if headers.has_key('status'):
            del headers['status']
        for key, val in headers.items():
            if key=='content-type':
                self.mod_python_request.content_type=val
            key, val = self.fixHeader(key, val)
            self.mod_python_request.headers_out[key]=val
        for name, attrs in self.cookies.items():
            cookie = '%s="%s"' % (name, attrs['value'])
            for name, v in attrs.items():
                name = name.lower()
                if name == 'expires':
                    cookie = '%s; Expires=%s' % (cookie,v)
                elif name == 'domain':
                    cookie = '%s; Domain=%s' % (cookie,v)
                elif name == 'path':
                    cookie = '%s; Path=%s' % (cookie,v)
                elif name == 'max_age':
                    cookie = '%s; Max-Age=%s' % (cookie,v)
                elif name == 'comment':
                    cookie = '%s; Comment=%s' % (cookie,v)
                elif name == 'secure' and v:
                    cookie = '%s; Secure' % cookie
            self.mod_python_request.headers_out.add('Set-Cookie',cookie)
        for key, val in self.multi_headers:
            key, val = self.fixHeader(key, val)
            self.mod_python_request.headers_out.add(key,val)
        self.mod_python_request.write(body)

    def write(self,data):
        """\
        Return data as a stream

        HTML data may be returned using a stream-oriented interface.
        This allows the browser to display partial results while
        computation of a response to proceed.

        The published object should first set any output headers or
        cookies on the response object.

        Note that published objects must not generate any errors
        after beginning stream-oriented output.

        """
        if not self._wrote:
            self.outputBody()
            self._wrote = 1

        self.mod_python_request.write(data)


